#!/usr/bin/wish -f


# Function for use as commands
#-----------------------------


# Skunk mit jcp laden
proc jcp_go {filename overwrite overwrite_address skiphdr skip_bytes skipboot isflash flash word_mode} {
   set line ""
   set command "jcp"
   if {$isflash == 1} {
      if {$word_mode} {
         append command " -wf"
      }
      else {
         append command "-f"
      }
      switch $flash {
         bank1 {
            set command $command
         }
         bank2 {
            append command " -2"
         }
         6mb {
            append command " -6"
         }
      }
   }
   if {$skipboot == 1} {
      append command " -n"
   }
   if {$skiphdr == 1} {
      append command " -h "
      append command $skip_bytes
   }
   if {$overwrite == 1} {
      append command " -o "
      append command $overwrite_address
   }
   append command " "
   append command $filename
   set io [open |$command]
   while {[gets $io this] >=0} {
      append line $this
      append line "\n"
   }
   set line [string map {"\b" "\n"} $line]
   return $line
}


# Seriennummer des Skunk Boards ermitteln
proc jcp_serial {} {
   set line ""
   set command "jcp"
   append command " -s"
   set io [open |$command]
   while {[gets $io this] >=0} {
      append line $this
      append line "\n"
   }
   set line [string map {"\b" "\n"} $line]
   return $line
}


# Skunk Boards reseten
proc jcp_reset {} {
   set line ""
   set command "jcp"
   append command " -r"
   set io [open |$command]
   while {[gets $io this] >=0} {
      append line $this
      append line "\n"
   }
   set line [string map {"\b" "\n"} $line]
   return $line
}


# ROM Bank dumpen
proc jcp_dump {bank} {
   set line ""
   set save_filename [tk_getSaveFile]
   set len [string length $save_filename]
   if {$len != 0} {
      set command "jcp"
      if {bank == 2} {
         append command " -2"
      }
      append command " -d "
      append command $save_filename
      set io [open |$command]
      while {[gets $io this] >=0} {
         append line $this
         append line "\n"
      }
      set line [string map {"\b" "\n"} $line]
   }
   return $line
}


# update skunk
proc jcp_update {} {
   set line ""
   set firmware_filename [tk_getOpenFile];
   set len [string length $firmware_filename]
   if {$len != 0} {
      set command "jcp"
      append command " -u "
      append command $firmware_filename
      set io [open |$command]
      while {[gets $io this] >=0} {
         append line $this
         append line "\n"
      }
      set line [string map {"\b" "\n"} $line]
   }
   return $line;
}


# GUI Design
#-----------


option add *tearOff 0

# Initialisierung
set flash bank1

wm title . "SkunkGUI"

# Hauptmenue
#-----------
menu .m_menubar
. configure -menu .m_menubar
menu .m_menubar.file
menu .m_menubar.tools
menu .m_menubar.help
.m_menubar add cascade -menu .m_menubar.file -label File
.m_menubar add cascade -menu .m_menubar.tools -label Tools
.m_menubar add cascade -menu .m_menubar.help -label Help
# Submenues unter "File"
.m_menubar.file add command -label "Dump ROM Bank 1 to File" -command {jcp_dump 1}
# Ctrl+1
.m_menubar.file add command -label "Dump ROM Bank 2 to File" -command {jcp_dump 2}
# Ctrl+2
.m_menubar.file add separator
.m_menubar.file add command -label "Quit" -command "exit 0"
# Submenues unter "File"
.m_menubar.tools add command -label "Reset Skunk" -command {set output [jcp_reset]; .f_actions.p_output.t_output insert 1.0 $output}
# Ctrl+R
.m_menubar.tools add command -label "Get Skunk Serial" -command {set output [jcp_serial]; .f_actions.p_output.t_output insert 1.0 $output}
# Ctrl+S
.m_menubar.tools add command -label "Update Skunk"
# Ctrl+U
.m_menubar.tools add separator
#.m_menubar.tools add checkbutton -label "Rev 1 Skunk Board" -variable rev_1 -onvalue 1 -offvalue 0
.m_menubar.tools add checkbutton -label "Use Word Flash Mode" -variable word_mode -onvalue 1 -offvalue 0
# Submenues unter "Help"
.m_menubar.help add command -label "About SkunkGUI" -command {tk_messageBox -message "SkunkGUI V0.1 Linux TK/TCL\n\nCopyright (c) Michael Bernstein\n\nSkunkGUI is a graphical front end for skunkboard users. SkunkGUI simplifies using your skunk board with your Jaguar"}
# Ctrl+A
#.m_menubar.help add separator
#.m_menubar.help add command -label "Hillsoftware Home Page"
#.m_menubar.help add command -label "Download Page"

# Arbeitsbereich
#---------------
frame .f_actions
.f_actions configure -borderwidth 2 -relief sunken

# Bereich mit Dateiauswahl
#-------------------------
frame .f_actions.f_file
button .f_actions.f_file.b_select -text "Select File" -command {set filename [tk_getOpenFile];set len [string length $filename];if {$len != 0} {.f_actions.f_file.b_go configure -state normal}}
entry .f_actions.f_file.e_file -textvariable filename
button .f_actions.f_file.b_go -text "Go" -state disabled -command {set output [jcp_go $filename $overwrite $overwrite_address $skiphdr $skip_bytes $skipboot $isflash $flash $word_mode]; .f_actions.p_output.t_output insert 1.0 $output}

# Bereich mit Einstellungen
#--------------------------
labelframe .f_actions.p_option -text "Option"
frame .f_actions.p_option.f_option
frame .f_actions.p_option.f_reset
checkbutton .f_actions.p_option.f_option.c_overwrite -text "overwrite address" -variable overwrite -onvalue 1 -offvalue 0
entry .f_actions.p_option.f_option.e_address -textvariable overwrite_address
checkbutton .f_actions.p_option.f_option.c_skip_hdr -text "skip header bytes" -variable skiphdr -onvalue 1 -offvalue 0
entry .f_actions.p_option.f_option.e_bytes -textvariable skip_bytes
checkbutton .f_actions.p_option.f_option.c_skip_boot -text "skip boot" -variable skipboot -onvalue 1 -offvalue 0
button .f_actions.p_option.f_reset.b_reset -text "Reset Skunk" -command {set output [jcp_reset]; .f_actions.p_output.t_output insert 1.0 $output}
checkbutton .f_actions.p_option.f_option.c_is_flash -text "Is Flash?" -variable isflash -onvalue 1 -offvalue 0
radiobutton .f_actions.p_option.f_option.r_bank_1 -text "Bank 1" -variable flash -value bank1
radiobutton .f_actions.p_option.f_option.r_bank_2 -text "Bank 2" -variable flash -value bank2
radiobutton .f_actions.p_option.f_option.r_6_mb -text "6 MB" -variable flash -value 6mb

# Bereich mit Textausgaben des jcp
#---------------------------------
labelframe .f_actions.p_output -text "Output"
text .f_actions.p_output.t_output -width 75 -height 12

# Widgets positionieren
#----------------------
grid .f_actions -row 0 -column 0
grid .f_actions.f_file -row 1 -column 0 -sticky w -padx 1m
grid .f_actions.f_file.b_select -row 0 -column 0 -sticky w -padx 1m
grid .f_actions.f_file.e_file -row 0 -column 1 -sticky w -padx 1m
grid .f_actions.f_file.b_go -row 0 -column 2 -sticky w -padx 1m
grid .f_actions.p_option -row 2 -column 0 -padx 1m
grid .f_actions.p_option.f_option -row 0 -column 0 -padx 1m
grid .f_actions.p_option.f_reset -row 1 -column 0 -padx 1m
grid .f_actions.p_option.f_option.c_overwrite -row 0 -column 0 -sticky w -padx 1m
grid .f_actions.p_option.f_option.e_address -row 0 -column 1 -sticky w -padx 1m
grid .f_actions.p_option.f_option.c_skip_hdr -row 1 -column 0 -sticky w -padx 1m
grid .f_actions.p_option.f_option.e_bytes -row 1 -column 1 -sticky w -padx 1m
grid .f_actions.p_option.f_option.c_skip_boot -row 2 -column 0 -sticky w -padx 1m
grid .f_actions.p_option.f_reset.b_reset -row 0 -column 0 -padx 1m
grid .f_actions.p_option.f_option.c_is_flash -row 0 -column 4 -padx 1m
grid .f_actions.p_option.f_option.r_bank_1 -row 1 -column 4 -padx 1m
grid .f_actions.p_option.f_option.r_bank_2 -row 1 -column 5 -padx 1m
grid .f_actions.p_option.f_option.r_6_mb   -row 1 -column 6 -padx 1m
grid .f_actions.p_output -row 3 -column 0 -padx 1m
grid .f_actions.p_output.t_output -row 0 -column 0 -padx 1m
